// Author: Harm Fesefeldt (HarfeSoft) (cc) 2007

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;



public class Brille extends JApplet {

    Demo demo; 				
    DemoControls controls;
    
    public void init() {
        getContentPane().setLayout(new GridLayout(1,1)); 
        resize(800,400);
        getContentPane().add(demo = new Demo());
        controls = new DemoControls(demo);
	controls.show();
	
    }
	

    public void start() {
	
        demo.start();
    }
  
    public void stop() {
        demo.stop();
    }




    static class Demo extends JPanel implements Runnable {
    
        private static Color colors[] = 
                { Color.blue,Color.red,Color.gray,Color.green};
        private Thread thread;
        private BufferedImage bimg;
        private long now, deltaT, lasttime;
        private boolean active;
        private boolean clearDemo = true;
        protected Ball balls[] = new Ball[1];
        protected Linse linsen[] = new Linse[2];
        protected Rohr rohre[] = new Rohr[2];
        public boolean textdraw = false;
        Font font;
        public int gegenweite, bildweite, dAB;
        int ixb1, iyb1, ixb2, iyb2;
        int ir1x, ir1y, ir0x, ir0y, il1x, il1y, il2x, il2y;
    

        public Demo() {
            setBackground(Color.black);
            
                balls[0] = new Ball(colors[2], 140);
                linsen[0] = new Linse(colors[0], 200,  23, 60);
                linsen[1] = new Linse(colors[3], 250,   0, 30);
                rohre[0] = new Rohr(colors[1], 50, 50,10);
                rohre[1] = new Rohr(colors[1], 50, 50,10);
	        for (int i = 0; i < linsen.length; i++) linsen[i].isSelected = true;
	        for (int i = 0; i < balls.length; i++) balls[i].isSelected = true;
	        for (int i = 0; i < rohre.length; i++) rohre[i].isSelected = true;
                font = new Font("Arial",Font.BOLD,18);
                active = true;
                gegenweite = 400;
                dAB = 10;
                linsen[1].isSelected = false;
            
        }
        
        
        public void step(int w, int h) {
            if (lasttime == 0) {
                lasttime = System.currentTimeMillis();
            }
            now = System.currentTimeMillis();
            deltaT = now - lasttime;
            for (int i = 0; i < balls.length; i++) {
                balls[i].step(deltaT, w, h);
            }
            for (int i = 0; i < linsen.length; i++) {
                linsen[i].step(deltaT, w, h);
            }
            for (int i = 0; i < rohre.length; i++) {
                rohre[i].step(deltaT, w, h);
            }
            if (!active) {
                active = true;
                for (int i = 0; i < balls.length; i++) {
                    balls[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    balls[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    balls[i].Ay = 0.0002f;
                }
                for (int i = 0; i < linsen.length; i++) {
                    linsen[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    linsen[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    linsen[i].Ay = 0.0002f;
                }
                for (int i = 0; i < rohre.length; i++) {
                    rohre[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    rohre[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    rohre[i].Ay = 0.0002f;
                }
            }
        }
    
    
        public void drawDemo(int w, int h, Graphics2D g2) {
            for (int i = 0; i < balls.length; i++) {
                Ball b = balls[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < linsen.length; i++) {
                Linse b = linsen[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < rohre.length; i++) {
                Rohr b = rohre[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            lasttime = now;
        }
    
    
        public Graphics2D createGraphics2D(int w, int h) {
            Graphics2D g2 = null;
            if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
                bimg = (BufferedImage) createImage(w, h);
            } 
            g2 = bimg.createGraphics();
            if (clearDemo) {
                g2.setBackground(getBackground());
                g2.clearRect(0, 0, w, h);
            }
            return g2;
        }
    
    
        public void paint(Graphics g) {
            Dimension d = getSize();
            step(d.width, d.height);
            Graphics2D g2 = createGraphics2D(d.width, d.height);
            drawDemo(d.width, d.height, g2);
            g2.dispose();
            g.drawImage(bimg, 0, 0, this);
            
              g.setColor(Color.white);
              g.drawLine(0, d.height/2, d.width, d.height/2);
              g.drawLine(0, d.height/2+1, d.width, d.height/2+1);
              g.drawLine(0, d.height/2-1, d.width, d.height/2-1);

              g.setColor(Color.yellow);

              g.drawLine(ir0x, ir0y, il2x, il2y);
              g.drawLine(il2x, il2y, ir1x, ir1y);

              g.setFont(font);
              g.setColor(Color.white);
              g.drawString("H.Fesefeldt (cc) 2007", 25, d.height-25); 
              g.drawString("g = " + gegenweite + " mm", 200, 25); 
              g.drawString("b = " + bildweite + " mm", 600, 25); 
            
        }
    
    
        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            reset();
            thread.start();
        }
    
        public void reset() {
            linsen[0].Ax = 0.f;
            linsen[0].Ay = 0.f;
            linsen[0].Vx = 0.f;
            linsen[0].Vy = 0.f;
            linsen[1].Ax = 0.f;
            linsen[1].Ay = 0.f;
            linsen[1].Vx = 0.f;
            linsen[1].Vy = 0.f;
            balls[0].Ax = 0.f;
            balls[0].Ay = 0.f;
            balls[0].Vx = 0.f;
            balls[0].Vy = 0.f;
            rohre[0].Ax = 0.f;
            rohre[0].Ay = 0.f;
            rohre[0].Vx = 0.f;
            rohre[0].Vy = 0.f;
            rohre[1].Ax = 0.f;
            rohre[1].Ay = 0.f;
            rohre[1].Vx = 0.f;
            rohre[1].Vy = 0.f;
            rohre[0].isSelected = true;
            rohre[1].isSelected = true;

            Dimension d = getSize();
            balls[0].x = d.width - balls[0].bsize-50;
            balls[0].y = d.height/2 - balls[0].bsize/2;
            linsen[0].x = d.width - balls[0].bsize-50;
            linsen[0].y = d.height/2 - linsen[0].bsizey/2;
            linsen[1].x = d.width - balls[0].bsize-80;
            linsen[1].y = d.height/2 - linsen[1].bsizey/2;
            double f2 = 0.;
            if (linsen[1].brennweite == 0) f2 = linsen[0].brennweite;
            else { 
              f2 = 1./linsen[0].brennweite + 1./linsen[1].brennweite
                   - 1.*dAB/(linsen[0].brennweite*linsen[1].brennweite);
              f2 = 1./f2;
            }
            double f1 = f2/1.34;
            double g = gegenweite;
            double G = rohre[0].height1;
            double gf1 = g - f1;
            if(gf1 == 0.) gf1 = 1.;
            double b = f2*g/gf1;
            double B = Math.abs(b*G/g);
            if(B > d.height/20.) B = d.height/20. - 0.5;
            rohre[1].makeImages((int)(10*B), (int)(10*B), 10);          
            rohre[0].x = linsen[0].x + linsen[0].bsizex/2 - (float)(g) - rohre[0].width;
            rohre[0].y = d.height/2 - rohre[0].height1;
            rohre[1].x = linsen[0].x + linsen[0].bsizex/2 + (float)(10*b);
            rohre[1].y = d.height/2;
            active = true;
            ixb1 = (int)(linsen[0].x + linsen[0].bsizex/2 - f1);
            iyb1 = (int)(d.height/2);
            ixb2 = (int)(linsen[0].x + linsen[0].bsizex/2 + 10*f2);
            iyb2 = iyb1;
            ir0x = (int)(rohre[0].x + rohre[0].bsizex/2);
            ir0y = (int)(rohre[0].y);
            ir1x = (int)(rohre[1].x + rohre[1].bsizex/2);
            ir1y = (int)(rohre[1].y + rohre[1].bsizey);
            il1x = (int)(linsen[0].x + linsen[0].width/2);
            il1y = ir0y;
            il2x = il1x;
            il2y = d.height/2;
            bildweite = (int)(b);
            if( rohre[0].x < 0.f) rohre[0].isSelected = false;
            if( rohre[1].x > d.width) rohre[1].isSelected = false;
        }    
    
        public synchronized void stop() {
            thread = null;
        }
    
    
        public void run() {
            Thread me = Thread.currentThread();
            while (thread == me) {
                repaint();
                try {
                    thread.sleep(10);
                } catch (InterruptedException e) { break; }
            }
            thread = null;
        }
    }


        static class Linse {
        
            public int height, brennweite, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Linse(Color color, int height, int brennweite, int width) {
                this.color = color;
                this.height = height;
                this.brennweite = brennweite;
                this.width = width;
                makeImages(height, brennweite, width);
            }
        
        
            public void makeImages(int height, int brennweite, int width) {
                this.height = height;
                this.brennweite = brennweite;
                this.width   = width;
                bsizex = width;
                bsizey = height;
                byte[] data = new byte[height * width];
                int maxr = 0;
                int R = (height*height - width*width)/(4*width);
                if(brennweite <0) R = (height*height - 4*width*width/9)/(8*width/3);
                int p = 0;
                int x0 = 0;
                int y0 = height/2;
                for (int Y = 0; Y < height; Y++) {
                    
                    for (int X = 0; X < width; X++) {
                        if(brennweite > 0) {
                          x0 = R;                        
                          if(X > width/2) x0 = width - R;
                        }
                        else {
                          x0 = 2*width/3 + R;
                          if(X < width/2) x0 = width/3 - R; 
                        } 
                        int r = (int)(Math.sqrt((X-width/2.)*(X-width/2)+(Y-height/2)*(Y-height/2)));
                        if (r > maxr) {
                            maxr = r;
                        }
                        if (brennweite > 0) {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) < R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        else {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) > R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Linse class


        static class Rohr {
        
            public int height1, height2, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Rohr(Color color, int height1, int height2, int width) {
                this.color = color;
                makeImages(height1, height2, width);
            }
        
        
            public void makeImages(int height1, int height2, int width) {
                this.height1 = height1;
                this.height2 = height2;
                this.width   = width;
                bsizex = width;
                bsizey = height1;
                byte[] data = new byte[height1 * width];
                int maxr = 0;
                int p = 0;
                for (int Y = 0; Y < height1; Y++) {
                    int x0 = 0;
                    if(Y < height1/2) x0 = (int) (width*Y*2./(height1-height2));
                    else              x0 = (int) (width*(height1-Y)*2./(height1-height2));
                    for (int X = 0; X < width; X++) {
                        int r = 0;
                        if( height1 < width) r = (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5);
                        else if (height1 > width) r = (int) (width/2 - Math.abs(X - width/2.)   + 0.5);
                        else { 
                            r = Math.max( (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5),
                                          (int) (width/2 - Math.abs(X - width/2.)   + 0.5)   );
                            if(X > 0.75*width) r = (int)(r*Math.exp(-0.75*(X-0.75*width)));
                            if(Y > 0.75*width) r = (int)(r*Math.exp(-0.75*(Y-0.75*width)));
                        }
                        if (r > maxr) {
                            maxr = r;
                        }
                        if(X < x0 ) data[p] = r <= 0 ? 1 : (byte) r;
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height1,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Rohr class

        static class Ball {
        
            public int bsize;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;

        
            public Ball(Color color, int bsize) {
                this.color = color;
                makeImages(bsize);
            }
        
        
            public void makeImages(int bsize) {
                this.bsize = bsize*2;
                int R = bsize;
                byte[] data = new byte[R * 2 * R * 2];
                int maxr = 0;
                for (int Y = 2 * R; --Y >= 0;) {
                    int x0 = (int) (Math.sqrt(R * R - (Y - R) * (Y - R)) + 0.5);
                    int p = Y * (R * 2) + R - x0;
                    for (int X = -x0; X < x0; X++) {
                        int x = X + 15;
                        int y = Y - R + 15;
                        int r = (int) (Math.sqrt(x * x + y * y) + 0.5);
                        if (r > maxr) {
                            maxr = r;
                        }
                        data[p++] = r <= 0 ? 1 : (byte) r;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        R*2,R*2,R*2,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsize >= w) {
                    x = w - bsize;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsize >= h) {
                    y = h - bsize;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Ball class


    static class DemoControls extends JFrame implements ActionListener {

        Demo demo;

	JButton reset;
        JButton start;
        JButton label1;
        JButton label2;
        JButton label3;
        JButton label4;
        JComboBox fau;
        JComboBox fbr;
        JComboBox dau;
        JComboBox gel;


        public DemoControls(Demo demo) {
            this.demo = demo;
	 
	    resize(800,150);
	    getContentPane().setLayout(new GridLayout(2,5));
            setBackground(Color.gray);

            label1 = new JButton("f(Auge) =");
	    getContentPane().add(label1);
            getContentPane().add(fau = new JComboBox());
            fau.addItem("17");
            fau.addItem("18");
            fau.addItem("19");
            fau.addItem("20");
            fau.addItem("21");
            fau.addItem("22");
            fau.addItem("23");
            fau.addItem("24");
            fau.addItem("25");
            fau.addItem("26");
            fau.setSelectedIndex(6);
            fau.addActionListener(this);
            label3 = new JButton("D(Brille)/Dpt =");
	    getContentPane().add(label3);
            getContentPane().add(fbr = new JComboBox());
            fbr.addItem("-10");
            fbr.addItem("-9");
            fbr.addItem("-8");
            fbr.addItem("-7");
            fbr.addItem("-6");
            fbr.addItem("-5");
            fbr.addItem("-4");
            fbr.addItem("-3");
            fbr.addItem("-2");
            fbr.addItem("-1");
            fbr.addItem("0");
            fbr.addItem("1");
            fbr.addItem("2");
            fbr.addItem("3");
            fbr.addItem("4");
            fbr.addItem("5");
            fbr.addItem("6");
            fbr.addItem("7");
            fbr.addItem("8");
            fbr.addItem("9");
            fbr.addItem("10");
            fbr.setSelectedIndex(10);
            fbr.addActionListener(this);
	    start = new JButton("Start");
	    getContentPane().add(start);
	    start.addActionListener(this);
            label2 = new JButton("DM(Auge) =");
	    getContentPane().add(label2);
            getContentPane().add(dau = new JComboBox());
            dau.addItem("22");
            dau.addItem("23");
            dau.addItem("24");
            dau.addItem("25");
            dau.addItem("26");
            dau.setSelectedIndex(2);
            dau.addActionListener(this);
            label4 = new JButton("g =");
	    getContentPane().add(label4);
            getContentPane().add(gel = new JComboBox());
            gel.addItem("50");
            gel.addItem("75");
            gel.addItem("100");
            gel.addItem("150");
            gel.addItem("200");
            gel.addItem("250");
            gel.addItem("300");
            gel.addItem("350");
            gel.addItem("400");
            gel.addItem("1000");
            gel.setSelectedIndex(8);
            gel.addActionListener(this);
	    reset = new JButton("Reset");
	    reset.addActionListener(this);
	    getContentPane().add(reset);
        }
         

        


        public void actionPerformed(ActionEvent e) { 
	    JComponent c = (JComponent) e.getSource();

	    
	    if(c== reset)
	    {

	      demo.reset();
	    }
            else if(c == start)
            {
              demo.active = false;
            }
            else if(c == fau) 
            {
                int size = Integer.parseInt((String) fau.getSelectedItem());
                demo.linsen[0].brennweite = size;
                demo.linsen[0].makeImages(200, size, 83 - size);
                demo.reset();
                return;
            }   
            else if(c == fbr) 
            {
                int size = Integer.parseInt((String) fbr.getSelectedItem());
                size = (int)(1000./size);
                demo.linsen[1].makeImages(250, size, 30);
                if(size == 0) demo.linsen[1].isSelected = false;
                else demo.linsen[1].isSelected = true;
                demo.reset();
                return;
            }                   
            else if(c == dau) 
            {
                int size = Integer.parseInt((String) dau.getSelectedItem());
                size = (int)(size*140./24);
                demo.balls[0].makeImages(size);
                demo.reset();
                return;

            }
            else if(c == gel) 
            {
                int size = Integer.parseInt((String) gel.getSelectedItem());
                demo.gegenweite = size;
                if(size > 500) demo.rohre[0].isSelected = false;
                else demo.rohre[0].isSelected = true;
                demo.reset();
                return;

            }
        }
    } // End DemoControls

} // End Auge class
