// Author: Harm Fesefeldt (HarfeSoft) (cc) 2007

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;



public class DoppelLinse extends JApplet {

    Demo demo; 				
    DemoControls controls;
    
    public void init() {
        getContentPane().setLayout(new GridLayout(1,1)); 
        resize(1000,600);
        getContentPane().add(demo = new Demo());
        controls = new DemoControls(demo);
	controls.show();
	
    }
	

    public void start() {
	
        demo.start();
    }
  
    public void stop() {
        demo.stop();
    }




    static class Demo extends JPanel implements Runnable {
    
        private static Color colors[] = 
                { Color.blue,Color.red,Color.green,Color.gray,Color.darkGray};
        private Thread thread;
        private BufferedImage bimg;
        private long now, deltaT, lasttime;
        private boolean active;
        private boolean clearDemo = true;
        protected Linse linsen[] = new Linse[2];
        protected Rohr rohre[] = new Rohr[4];
        public boolean textdraw = false;
        Font font;
        public int distance, GDraw, BDraw, ib1x, ib1y, ib2x, ib2y, ibx, iby, igx, igy, ih1x, ih2x;
        public double gegenweite, bildweite, brennweite, 
                      G, B, gf, xl0, xl1, xr0, xr1, xh1, xh2, gx, gy, bx, by;

        public Demo() {
            setBackground(Color.black);
            {
                linsen[0] = new Linse(colors[0], 400, -40, 70);
                linsen[1] = new Linse(colors[0], 500, 100, 50);
                rohre[0] = new Rohr(colors[1], 150, 150,10);
                rohre[1] = new Rohr(colors[2], 50,  50,10);
                rohre[2] = new Rohr(colors[3], 400,400,10);
                rohre[3] = new Rohr(colors[4], 400,400,10);
	        for (int i = 0; i < linsen.length; i++) linsen[i].isSelected = true;
	        for (int i = 0; i < rohre.length; i++) rohre[i].isSelected = true;
                font = new Font("Arial",Font.BOLD,18);
                active = true;
                gegenweite = 250;
                distance = 120;
            }
        }
        
        
        public void step(int w, int h) {
            if (lasttime == 0) {
                lasttime = System.currentTimeMillis();
            }
            now = System.currentTimeMillis();
            deltaT = now - lasttime;
            for (int i = 0; i < linsen.length; i++) {
                linsen[i].step(deltaT, w, h);
            }
            for (int i = 0; i < rohre.length; i++) {
                rohre[i].step(deltaT, w, h);
            }
            if (!active) {
                active = true;
                for (int i = 0; i < linsen.length; i++) {
                    linsen[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    linsen[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    linsen[i].Ay = 0.0002f;
                }
                for (int i = 0; i < rohre.length; i++) {
                    rohre[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    rohre[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    rohre[i].Ay = 0.0002f;
                }
            }
        }
    
    
        public void drawDemo(int w, int h, Graphics2D g2) {
            for (int i = 0; i < linsen.length; i++) {
                Linse b = linsen[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < rohre.length; i++) {
                Rohr b = rohre[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            lasttime = now;
        }
    
    
        public Graphics2D createGraphics2D(int w, int h) {
            Graphics2D g2 = null;
            if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
                bimg = (BufferedImage) createImage(w, h);
            } 
            g2 = bimg.createGraphics();
            if (clearDemo) {
                g2.setBackground(getBackground());
                g2.clearRect(0, 0, w, h);
            }
            return g2;
        }
    
    
        public void paint(Graphics g) {
            Dimension d = getSize();
            step(d.width, d.height);
            Graphics2D g2 = createGraphics2D(d.width, d.height);
            drawDemo(d.width, d.height, g2);
            g2.dispose();
            g.drawImage(bimg, 0, 0, this);
            
              g.setColor(Color.white);
              g.drawLine(0, d.height/2, d.width, d.height/2);
              g.drawLine(0, d.height/2+1, d.width, d.height/2+1);
              g.drawLine(0, d.height/2-1, d.width, d.height/2-1);

              g.drawLine(ib1x, ib1y-5, ib1x, ib1y+5); 
              g.drawLine(ib1x+1, ib1y-5, ib1x+1, ib1y+5); 
              g.drawLine(ib1x-1, ib1y-5, ib1x-1, ib1y+5); 
              g.drawLine(ib2x, ib2y-5, ib2x, ib2y+5); 
              g.drawLine(ib2x+1, ib2y-5, ib2x+1, ib2y+5); 
              g.drawLine(ib2x-1, ib2y-5, ib2x-1, ib2y+5); 


 
              g.setColor(Color.yellow);
              g.drawLine(igx, igy, ih2x, igy);
              g.drawLine(ih2x, igy, ibx, iby);
              g.drawLine(ibx, iby, ih1x, iby);
              g.drawLine(ih1x, iby, igx, igy);
              g.setFont(font);
              g.setColor(Color.white);
              g.drawString("H.Fesefeldt (cc) 2007", 25, d.height-25); 
              if(textdraw) {
                 g.drawString("g = " + (int)(gegenweite),  100,25);
                 g.drawString("G = " + (int)(G), 100,50);
                 g.drawString("f = " + (int)(brennweite+0.5), 400,25);
                 g.drawString("b = " + (int)(bildweite),  700,25);
                 g.drawString("B = " + (int)(B), 700,50);
                 g.drawString("h1 = " + (int)(xh1-xl0),  400,d.height - 25);
                 g.drawString("h2 = " + (int)(xh2-xl1),  500,d.height - 25);
              }            
        }
    
    
        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            reset();
            thread.start();
        }
    
        public void reset() {
            for(int i=0; i < linsen.length; i++) {
              linsen[i].Ax = 0.f;
              linsen[i].Ay = 0.f;
              linsen[i].Vx = 0.f;
              linsen[i].Vy = 0.f;
            }
            for(int i=0; i < rohre.length; i++) {
              rohre[i].Ax = 0.f;
              rohre[i].Ay = 0.f;
              rohre[i].Vx = 0.f;
              rohre[i].Vy = 0.f;
            }

            Dimension d = getSize();
            xl0 = d.width/2 - 100;
            linsen[0].x = (float)(xl0) - linsen[0].bsizex/2;
            linsen[0].y = d.height/2 - linsen[0].bsizey/2;
            xl1 = xl0 + distance;
            linsen[1].x = (float)(xl1) - linsen[1].bsizex/2;
            linsen[1].y = d.height/2 - linsen[1].bsizey/2;
            brennweite = 1./linsen[0].brennweite + 1./linsen[1].brennweite 
                         - (double)(distance)/(linsen[0].brennweite*linsen[1].brennweite);
            brennweite = 1./brennweite;
            xh1 = xl0 + brennweite*distance/linsen[1].brennweite;
            xh2 = xl1 - brennweite*distance/linsen[0].brennweite;
            rohre[2].x = (float)(xh1) - rohre[2].width/2;
            rohre[2].y = d.height/2 - rohre[2].height1/2;
            rohre[3].x = (float)(xh2) - rohre[3].width/2;
            rohre[3].y = d.height/2 - rohre[3].height1/2;           
            G = rohre[0].height1;
            GDraw = (int)(G);
            gf = gegenweite - brennweite;
            if(gf == 0) gf = 1;
            bildweite = brennweite*gegenweite/gf;
            B = bildweite*G/gegenweite;
            BDraw = (int)(Math.abs(B));
            if(BDraw > d.height/2) BDraw = d.height/2 -2;
            if(BDraw < 2) BDraw = 2;
            rohre[1].makeImages(BDraw, BDraw, 10);
            gx = xh1 - gegenweite; 
            gy = d.height/2 - rohre[0].height1;         
            rohre[0].x = (float)(gx) - rohre[0].width/2;
            rohre[0].y = (float)(gy);
            if(B > 0.) {
              bx = xh2 + bildweite;
              rohre[1].x = (float)(bx) - rohre[1].width/2;
              rohre[1].y = d.height/2;
              by = d.height/2 + rohre[1].height1;
            }
            else {
              bx = xh2 + bildweite;
              by = d.height/2 - rohre[1].height1;
              rohre[1].x = (float)(bx) - rohre[1].width/2;
              rohre[1].y = (float)(by);
            }
            ib1x = (int)(xh1 - brennweite);
            ib2x = (int)(xh2 + brennweite);
            ib1y = d.height/2;
            ib2y = ib1y;
            igx = (int)(gx);
            igy = (int)(gy);
            ibx = (int)(bx);
            iby = (int)(by);
            ih1x = (int)(xh1);
            ih2x = (int)(xh2);

            rohre[1].isSelected = true;
            if(rohre[1].x < 0.f || rohre[1].x > d.width) rohre[1].isSelected = false;
            active = true;
        }    
    
        public synchronized void stop() {
            thread = null;
        }
    
    
        public void run() {
            Thread me = Thread.currentThread();
            while (thread == me) {
                repaint();
                try {
                    thread.sleep(10);
                } catch (InterruptedException e) { break; }
            }
            thread = null;
        }
    }


        static class Linse {
        
            public int height, brennweite, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Linse(Color color, int height, int brennweite, int width) {
                this.color = color;
                this.brennweite = brennweite;
                this.width = width;
                makeImages(height, brennweite, width);
            }
        
        
            public void makeImages(int height, int brennweite, int width) {
                this.height = height;
                this.brennweite = brennweite;
                this.width   = width;
                bsizex = width;
                bsizey = height;
                byte[] data = new byte[height * width];
                int maxr = 0;
                int R = (height*height - width*width)/(4*width);
                if(brennweite <0) R = (height*height - 4*width*width/9)/(8*width/3);
                int p = 0;
                int x0 = 0;
                int y0 = height/2;
                for (int Y = 0; Y < height; Y++) {
                    
                    for (int X = 0; X < width; X++) {
                        if(brennweite > 0) {
                          x0 = R;                        
                          if(X > width/2) x0 = width - R;
                        }
                        else {
                          x0 = 2*width/3 + R;
                          if(X < width/2) x0 = width/3 - R; 
                        } 
                        int r = (int)(Math.sqrt((X-width/2.)*(X-width/2)+(Y-height/2)*(Y-height/2)));
                        if (r > maxr) {
                            maxr = r;
                        }
                        if (brennweite > 0) {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) < R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        else {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) > R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Linse class
        static class Rohr {
        
            public int height1, height2, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Rohr(Color color, int height1, int height2, int width) {
                this.color = color;
                makeImages(height1, height2, width);
            }
        
        
            public void makeImages(int height1, int height2, int width) {
                this.height1 = height1;
                this.height2 = height2;
                this.width   = width;
                bsizex = width;
                bsizey = height1;
                byte[] data = new byte[height1 * width];
                int maxr = 0;
                int p = 0;
                for (int Y = 0; Y < height1; Y++) {
                    int x0 = 0;
                    if(Y < height1/2) x0 = (int) (width*Y*2./(height1-height2));
                    else              x0 = (int) (width*(height1-Y)*2./(height1-height2));
                    for (int X = 0; X < width; X++) {
                        int r = 0;
                        if( height1 < width) r = (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5);
                        else if (height1 > width) r = (int) (width/2 - Math.abs(X - width/2.)   + 0.5);
                        else { 
                            r = Math.max( (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5),
                                          (int) (width/2 - Math.abs(X - width/2.)   + 0.5)   );
                            if(X > 0.75*width) r = (int)(r*Math.exp(-0.75*(X-0.75*width)));
                            if(Y > 0.75*width) r = (int)(r*Math.exp(-0.75*(Y-0.75*width)));
                        }
                        if (r > maxr) {
                            maxr = r;
                        }
                        if(X < x0 ) data[p] = r <= 0 ? 1 : (byte) r;
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height1,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Rohr class


    static class DemoControls extends JFrame implements ActionListener {

        Demo demo;

	JButton reset;
        JButton start;
        JButton text;
        JButton label1;
        JButton label2;
        JButton label3;
        JButton label4;
        JComboBox gegen;
        JComboBox dist;
        JComboBox f1;
        JComboBox f2;


        public DemoControls(Demo demo) {
            this.demo = demo;
	 
	    resize(800,100);
	    getContentPane().setLayout(new GridLayout(1,8));
            setBackground(Color.gray);


            label1 = new JButton("f1 =");
	    getContentPane().add(label1);
            getContentPane().add(f1 = new JComboBox());
            f1.addItem("-100");
            f1.addItem("-80");
            f1.addItem("-60");
            f1.addItem("-40");
            f1.addItem("-20");
            f1.setSelectedIndex(3);
            f1.addActionListener(this);
            label2 = new JButton("f2 =");
	    getContentPane().add(label2);
            getContentPane().add(f2 = new JComboBox());
            f2.addItem("20");
            f2.addItem("40");
            f2.addItem("60");
            f2.addItem("80");
            f2.addItem("100");
            f2.setSelectedIndex(4);
            f2.addActionListener(this);
            label3 = new JButton("d =");
	    getContentPane().add(label3);
            getContentPane().add(dist = new JComboBox());
            dist.addItem("10");
            dist.addItem("20");
            dist.addItem("40");
            dist.addItem("60");
            dist.addItem("80");
            dist.addItem("100");
            dist.addItem("120");
            dist.addItem("140");
            dist.setSelectedIndex(6);
            dist.addActionListener(this);
            label4 = new JButton("g =");
	    getContentPane().add(label4);
            getContentPane().add(gegen = new JComboBox());
            gegen.addItem("100");
            gegen.addItem("150");
            gegen.addItem("200");
            gegen.addItem("250");
            gegen.addItem("300");
            gegen.addItem("350");
            gegen.addItem("400");
            gegen.setSelectedIndex(3);
            gegen.addActionListener(this);

	    reset = new JButton("Reset");
	    reset.addActionListener(this);
	    getContentPane().add(reset);
	    start = new JButton("Start");
	    start.addActionListener(this);
	    getContentPane().add(start);
	    text = new JButton("Text");
	    text.addActionListener(this);
	    getContentPane().add(text);
        }
         

        


        public void actionPerformed(ActionEvent e) { 
	    JComponent c = (JComponent) e.getSource();

	    
	    if(c== reset)
	    {
	      demo.reset();
	    }
            else if(c == start)
            {
              demo.active = false;
            }
            else if(c == text)
            {
              if(demo.textdraw == false) demo.textdraw = true;
              else demo.textdraw = false;
            }
            else if(c == f1) 
            {
                int size = Integer.parseInt((String) f1.getSelectedItem());
                demo.linsen[0].brennweite = size;
                int brenn = 50 + (int)(200./size);
                demo.linsen[0].makeImages(400, size, brenn);
                demo.reset();
                return;
            }           
            else if(c == f2) 
            {
                int size = Integer.parseInt((String) f2.getSelectedItem());
                demo.linsen[1].brennweite = size;
                int brenn = 50 + (int)(200./size);
                demo.linsen[1].makeImages(400, size, brenn);
                demo.reset();
                return;

            }           
            else if(c == gegen) 
            {
                int size = Integer.parseInt((String) gegen.getSelectedItem());
                demo.gegenweite = size;
                demo.reset();
                return;
            }
            else if(c == dist) 
            {
                int size = Integer.parseInt((String) dist.getSelectedItem());
                demo.distance = size;
                demo.reset();
                return;
            }
        }
    } // End DemoControls

} // End linsen class
