// Author: Harm Fesefeldt (HarfeSoft) (cc) 2007

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;



public class Linsen extends JApplet {

    Demo demo; 				
    DemoControls controls;
    
    public void init() {
        getContentPane().setLayout(new GridLayout(1,1)); 
        resize(800,600);
        getContentPane().add(demo = new Demo());
        controls = new DemoControls(demo);
	controls.show();
	
    }
	

    public void start() {
	
        demo.start();
    }
  
    public void stop() {
        demo.stop();
    }




    static class Demo extends JPanel implements Runnable {
    
        private static Color colors[] = 
                { Color.blue,Color.red,Color.green};
        private Thread thread;
        private BufferedImage bimg;
        private long now, deltaT, lasttime;
        private boolean active;
        private boolean clearDemo = true;
        protected Linse linsen[] = new Linse[1];
        protected Rohr rohre[] = new Rohr[2];
        public boolean textdraw = false;
        Font font;
        public int gegenweite, bildweite, GDraw, BDraw;
        private int ir0x, ir0y, ir1x, ir1y, il1x, il1y, il2x, il2y, ibx, iby;


        public Demo() {
            setBackground(Color.black);
            {
                linsen[0] = new Linse(colors[0], 400, 100, 52);
                rohre[0] = new Rohr(colors[1], 100, 100,10);
                rohre[1] = new Rohr(colors[2], 50,  50,10);
	        for (int i = 0; i < linsen.length; i++) linsen[i].isSelected = true;
	        for (int i = 0; i < rohre.length; i++) rohre[i].isSelected = true;
                font = new Font("Arial",Font.BOLD,18);
                active = true;
                gegenweite = 300;
            }
        }
        
        
        public void step(int w, int h) {
            if (lasttime == 0) {
                lasttime = System.currentTimeMillis();
            }
            now = System.currentTimeMillis();
            deltaT = now - lasttime;
            for (int i = 0; i < linsen.length; i++) {
                linsen[i].step(deltaT, w, h);
            }
            for (int i = 0; i < rohre.length; i++) {
                rohre[i].step(deltaT, w, h);
            }
            if (!active) {
                active = true;
                for (int i = 0; i < linsen.length; i++) {
                    linsen[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    linsen[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    linsen[i].Ay = 0.0002f;
                }
                for (int i = 0; i < rohre.length; i++) {
                    rohre[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    rohre[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    rohre[i].Ay = 0.0002f;
                }
            }
        }
    
    
        public void drawDemo(int w, int h, Graphics2D g2) {
            for (int i = 0; i < linsen.length; i++) {
                Linse b = linsen[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < rohre.length; i++) {
                Rohr b = rohre[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            lasttime = now;
        }
    
    
        public Graphics2D createGraphics2D(int w, int h) {
            Graphics2D g2 = null;
            if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
                bimg = (BufferedImage) createImage(w, h);
            } 
            g2 = bimg.createGraphics();
            if (clearDemo) {
                g2.setBackground(getBackground());
                g2.clearRect(0, 0, w, h);
            }
            return g2;
        }
    
    
        public void paint(Graphics g) {
            Dimension d = getSize();
            step(d.width, d.height);
            Graphics2D g2 = createGraphics2D(d.width, d.height);
            drawDemo(d.width, d.height, g2);
            g2.dispose();
            g.drawImage(bimg, 0, 0, this);
            
              g.setColor(Color.white);
              g.drawLine(0, d.height/2, d.width, d.height/2);
              g.drawLine(0, d.height/2+1, d.width, d.height/2+1);
              g.drawLine(0, d.height/2-1, d.width, d.height/2-1);
              g.drawLine(d.width/2 - linsen[0].brennweite, d.height/2 - 5, 
                         d.width/2 - linsen[0].brennweite, d.height/2 + 5); 
              g.drawLine(d.width/2 + linsen[0].brennweite, d.height/2 - 5, 
                         d.width/2 + linsen[0].brennweite, d.height/2 + 5); 
              g.drawLine(d.width/2 - linsen[0].brennweite+1, d.height/2 - 5, 
                         d.width/2 - linsen[0].brennweite+1, d.height/2 + 5); 
              g.drawLine(d.width/2 + linsen[0].brennweite+1, d.height/2 - 5, 
                         d.width/2 + linsen[0].brennweite+1, d.height/2 + 5); 
              g.drawLine(d.width/2 - linsen[0].brennweite-1, d.height/2 - 5, 
                         d.width/2 - linsen[0].brennweite-1, d.height/2 + 5); 
              g.drawLine(d.width/2 + linsen[0].brennweite-1, d.height/2 - 5, 
                         d.width/2 + linsen[0].brennweite-1, d.height/2 + 5); 

 
              g.setColor(Color.yellow);
              g.drawLine(ir0x, ir0y, il1x, il1y);
              g.drawLine(il1x, il1y, ibx, iby);
              g.drawLine(ibx, iby, ir1x, ir1y);
              g.drawLine(ir1x, ir1y, il2x, il2y);
              g.drawLine(il2x, il2y, ir0x, ir0y);

              g.setFont(font);
              g.setColor(Color.white);
              g.drawString("H.Fesefeldt (cc) 2007", 25, d.height-25); 
              if(textdraw) {
                 g.drawString("g = " + gegenweite,  100,25);
                 g.drawString("G = " + GDraw, 100,50);
                 g.drawString("f = " + linsen[0].brennweite, 380,25);
                 g.drawString("b = " + bildweite,  600,25);
                 g.drawString("B = " + BDraw, 600,50);
              } 
            
        }
    
    
        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            reset();
            thread.start();
        }
    
        public void reset() {
            linsen[0].Ax = 0.f;
            linsen[0].Ay = 0.f;
            linsen[0].Vx = 0.f;
            linsen[0].Vy = 0.f;
            rohre[0].Ax = 0.f;
            rohre[0].Ay = 0.f;
            rohre[0].Vx = 0.f;
            rohre[0].Vy = 0.f;
            rohre[1].Ax = 0.f;
            rohre[1].Ay = 0.f;
            rohre[1].Vx = 0.f;
            rohre[1].Vy = 0.f;

            Dimension d = getSize();
            linsen[0].x = d.width/2 - linsen[0].bsizex/2;
            linsen[0].y = d.height/2 - linsen[0].bsizey/2;
            int f = linsen[0].brennweite;
            int g = gegenweite;
            int G = rohre[0].height1;
            GDraw = G;
            int gf = g - f;
            if(gf == 0) gf = 1;
            int b = f*g/gf;
            bildweite = b;
            int B = Math.abs(b*G/g);
            BDraw = B;
            if(f < 0 || g < f) ir1y = (int)(d.height/2 - B);
            else ir1y = (int)(d.height/2 + B);
            if(B > d.height/2) B = d.height/2 -2;
            if(f < 0) b = - Math.abs(b);
            rohre[1].makeImages(B, B, 10);          
            rohre[0].x = linsen[0].x + linsen[0].bsizex/2 - g - rohre[0].width;
            rohre[0].y = d.height/2 - rohre[0].height1;
            if(f >0) {
              rohre[1].x = linsen[0].x + linsen[0].bsizex/2 + b - rohre[1].width;
              rohre[1].y = d.height/2;
              if(g < f) rohre[1].y = d.height/2 - rohre[1].height1;
            }
            else {
              rohre[1].x = linsen[0].x + linsen[0].bsizex/2 + b - rohre[1].width/2;
              rohre[1].y = d.height/2 - rohre[1].height1;
            }
            rohre[1].isSelected = true;
            if(rohre[1].x < 0.f || rohre[1].x > d.width) rohre[1].isSelected = false;
            active = true;
            ir0x = (int)(rohre[0].x + rohre[0].width/2);
            ir0y = (int)(rohre[0].y);
            ir1x = (int)(rohre[1].x + rohre[1].width/2);

            ibx = (int)(d.width/2 + linsen[0].brennweite);
            iby = (int)(d.height/2);
            il1x = (int)(d.width/2);
            il1y = ir0y;
            il2x = (int)(d.width/2);
            il2y = (int)(d.height/2);
        }    
    
        public synchronized void stop() {
            thread = null;
        }
    
    
        public void run() {
            Thread me = Thread.currentThread();
            while (thread == me) {
                repaint();
                try {
                    thread.sleep(10);
                } catch (InterruptedException e) { break; }
            }
            thread = null;
        }
    }


        static class Linse {
        
            public int height, brennweite, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Linse(Color color, int height, int brennweite, int width) {
                this.color = color;
                this.brennweite = brennweite;
                this.width = width;
                makeImages(height, brennweite, width);
            }
        
        
            public void makeImages(int height, int brennweite, int width) {
                this.height = height;
                this.brennweite = brennweite;
                this.width   = width;
                bsizex = width;
                bsizey = height;
                byte[] data = new byte[height * width];
                int maxr = 0;
                int R = (height*height - width*width)/(4*width);
                if(brennweite <0) R = (height*height - 4*width*width/9)/(8*width/3);
                int p = 0;
                int x0 = 0;
                int y0 = height/2;
                for (int Y = 0; Y < height; Y++) {
                    
                    for (int X = 0; X < width; X++) {
                        if(brennweite > 0) {
                          x0 = R;                        
                          if(X > width/2) x0 = width - R;
                        }
                        else {
                          x0 = 2*width/3 + R;
                          if(X < width/2) x0 = width/3 - R; 
                        } 
                        int r = (int)(Math.sqrt((X-width/2.)*(X-width/2)+(Y-height/2)*(Y-height/2)));
                        if (r > maxr) {
                            maxr = r;
                        }
                        if (brennweite > 0) {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) < R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        else {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) > R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Linse class
        static class Rohr {
        
            public int height1, height2, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Rohr(Color color, int height1, int height2, int width) {
                this.color = color;
                makeImages(height1, height2, width);
            }
        
        
            public void makeImages(int height1, int height2, int width) {
                this.height1 = height1;
                this.height2 = height2;
                this.width   = width;
                bsizex = width;
                bsizey = height1;
                byte[] data = new byte[height1 * width];
                int maxr = 0;
                int p = 0;
                for (int Y = 0; Y < height1; Y++) {
                    int x0 = 0;
                    if(Y < height1/2) x0 = (int) (width*Y*2./(height1-height2));
                    else              x0 = (int) (width*(height1-Y)*2./(height1-height2));
                    for (int X = 0; X < width; X++) {
                        int r = 0;
                        if( height1 < width) r = (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5);
                        else if (height1 > width) r = (int) (width/2 - Math.abs(X - width/2.)   + 0.5);
                        else { 
                            r = Math.max( (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5),
                                          (int) (width/2 - Math.abs(X - width/2.)   + 0.5)   );
                            if(X > 0.75*width) r = (int)(r*Math.exp(-0.75*(X-0.75*width)));
                            if(Y > 0.75*width) r = (int)(r*Math.exp(-0.75*(Y-0.75*width)));
                        }
                        if (r > maxr) {
                            maxr = r;
                        }
                        if(X < x0 ) data[p] = r <= 0 ? 1 : (byte) r;
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height1,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Rohr class


    static class DemoControls extends JFrame implements ActionListener {

        Demo demo;

	JButton reset;
        JButton start;
        JButton label1;
        JButton label2;
        JButton text;
        JComboBox vel;
        JComboBox gel;


        public DemoControls(Demo demo) {
            this.demo = demo;
	 
	    resize(500,100);
	    getContentPane().setLayout(new GridLayout(1,5));
            setBackground(Color.gray);

	    reset = new JButton("Reset");
	    reset.addActionListener(this);
	    getContentPane().add(reset);
	    start = new JButton("Start");
	    start.addActionListener(this);
	    getContentPane().add(start);
	    text = new JButton("Text");
	    text.addActionListener(this);
	    getContentPane().add(text);
            label1 = new JButton("f =");
	    getContentPane().add(label1);
            getContentPane().add(vel = new JComboBox());
            vel.addItem("-100");
            vel.addItem("-80");
            vel.addItem("-60");
            vel.addItem("-40");
            vel.addItem("-20");
            vel.addItem("20");
            vel.addItem("40");
            vel.addItem("60");
            vel.addItem("80");
            vel.addItem("100");
            vel.setSelectedIndex(9);
            vel.addActionListener(this);
            label2 = new JButton("g =");
	    getContentPane().add(label2);
            getContentPane().add(gel = new JComboBox());
            gel.addItem("10");
            gel.addItem("20");
            gel.addItem("30");
            gel.addItem("50");
            gel.addItem("100");
            gel.addItem("150");
            gel.addItem("200");
            gel.addItem("300");
            gel.setSelectedIndex(7);
            gel.addActionListener(this);
        }
         

        


        public void actionPerformed(ActionEvent e) { 
	    JComponent c = (JComponent) e.getSource();

	    
	    if(c== reset)
	    {

	      demo.reset();
	    }
            else if(c == start)
            {
              demo.active = false;
            }
            else if(c == text)
            {
              if(demo.textdraw == false) demo.textdraw = true;
              else demo.textdraw = false;
            }
            else if(c == vel) 
            {
                int size = Integer.parseInt((String) vel.getSelectedItem());
                demo.linsen[0].brennweite = size;
                int brenn = 50 + (int)(200./size);
                demo.linsen[0].makeImages(400, size, brenn);
                demo.reset();
                return;

            }           
            else if(c == gel) 
            {
                int size = Integer.parseInt((String) gel.getSelectedItem());
                demo.gegenweite = size;
                demo.reset();
                return;

            }
        }
    } // End DemoControls

} // End linsen class
