// Author: Harm Fesefeldt (HarfeSoft) (cc) 2007

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;



public class MatOptAufgabe extends JApplet {

    Demo demo; 				
    DemoControls controls;
    
    public void init() {
        getContentPane().setLayout(new GridLayout(1,1)); 
        resize(1000,600);
        getContentPane().add(demo = new Demo());
        controls = new DemoControls(demo);
	controls.show();
	
    }
	

    public void start() {
	
        demo.start();
    }
  
    public void stop() {
        demo.stop();
    }




    static class Demo extends JPanel implements Runnable {
    
        private static Color colors[] = 
                { Color.blue,Color.red,Color.green, Color.gray, Color.magenta, Color.yellow};
        private Thread thread;
        private BufferedImage bimg;
        private long now, deltaT, lasttime;
        private boolean active;
        private boolean clearDemo = true;
        protected Linse linsen[] = new Linse[3];
        protected Rohr rohre[] = new Rohr[3];
        protected Line lines[][] = new Line[3][6];
        public boolean textdraw = false;
        Font font;
        public int distance1, distance2, GDraw, BDraw, ip1x, ip1y, ip2x, ip2y, ih;
        public double gegenweite, bildweite, G, B, r, rs, xl0, xl1, xl2, xh;


        public Demo() {
            setBackground(Color.black);
            {
                linsen[0] = new Linse(colors[0], 400,  250, 30);
                linsen[1] = new Linse(colors[0], 400,  -50, 30);
                linsen[2] = new Linse(colors[0], 400,   50, 30);
                rohre[0] = new Rohr(colors[1], 150, 150, 10);
                rohre[1] = new Rohr(colors[2], 50,  50, 10);
                rohre[2] = new Rohr(colors[3], 500, 500, 10);
                for (int i = 0; i < 3; i++) {
                   for(int j = 0; j < 6; j++) { 
                     lines[i][j] = new Line(colors[5]);
                   }
                }
	        for (int i = 0; i < linsen.length; i++) linsen[i].isSelected = true;
	        for (int i = 0; i < rohre.length; i++) rohre[i].isSelected = true;
           
                font = new Font("Arial",Font.BOLD,18);
                active = true;
                gegenweite = 5*60;
                distance1 = 200;
                distance2 = 250;
                xh = 50;
            }
        }
        
        
        public void step(int w, int h) {
            if (lasttime == 0) {
                lasttime = System.currentTimeMillis();
            }
            now = System.currentTimeMillis();
            deltaT = now - lasttime;
            for (int i = 0; i < linsen.length; i++) {
                linsen[i].step(deltaT, w, h);
            }
            for (int i = 0; i < rohre.length; i++) {
                rohre[i].step(deltaT, w, h);
            }
            if (!active) {
                active = true;
                for (int i = 0; i < linsen.length; i++) {
                    linsen[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    linsen[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    linsen[i].Ay = 0.0002f;
                }
                for (int i = 0; i < rohre.length; i++) {
                    rohre[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    rohre[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    rohre[i].Ay = 0.0002f;
                }
            }
        }
    
    
        public void drawDemo(int w, int h, Graphics2D g2) {
            for (int i = 0; i < linsen.length; i++) {
                Linse b = linsen[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < rohre.length; i++) {
                Rohr b = rohre[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            lasttime = now;
        }
    
    
        public Graphics2D createGraphics2D(int w, int h) {
            Graphics2D g2 = null;
            if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
                bimg = (BufferedImage) createImage(w, h);
            } 
            g2 = bimg.createGraphics();
            if (clearDemo) {
                g2.setBackground(getBackground());
                g2.clearRect(0, 0, w, h);
            }
            return g2;
        }
    
    
        public void paint(Graphics g) {
            Dimension d = getSize();
            step(d.width, d.height);
            Graphics2D g2 = createGraphics2D(d.width, d.height);
            drawDemo(d.width, d.height, g2);
            g2.dispose();
            g.drawImage(bimg, 0, 0, this);
            
              g.setColor(Color.white);
              g.drawLine(0, d.height/2, d.width, d.height/2);
              g.drawLine(0, d.height/2+1, d.width, d.height/2+1);
              g.drawLine(0, d.height/2-1, d.width, d.height/2-1);
 
              g.setColor(Color.yellow);
              for(int i = 0; i < 3; i++) {
                 for(int j = 0; j < 6; j++) {
                    ip1x = lines[i][j].ip1x;
                    ip1y = lines[i][j].ip1y;
                    ip2x = lines[i][j].ip2x;
                    ip2y = lines[i][j].ip2y;
                    g.setColor(lines[i][j].color);
                    g.drawLine(ip1x, ip1y, ip2x, ip2y);
                 }
              }


              g.setFont(font);
              g.setColor(Color.white);
              g.drawString("H.Fesefeldt (cc) 2007", 25, d.height-25); 
              if(textdraw) {
                 g.drawString("g = " + (int)(gegenweite/5),  100,25);
                 g.drawString("G = " + (int)(G/5), 100,50);
                 g.drawString("h = " + ih/5, 400, 25);
                 g.drawString("f = " + (ih/5+(int)(bildweite/5)), 400, 50);
                 g.drawString("b = " + (int)(bildweite/5),   700,25);
                 g.drawString("B = " + (int)(B/5), 700,50);
              }            
        }
    
    
        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            reset();
            thread.start();
        }
    
        public void reset() {
            Dimension d = getSize();

            for(int i=0; i < linsen.length; i++) {
              linsen[i].Ax = 0.f;
              linsen[i].Ay = 0.f;
              linsen[i].Vx = 0.f;
              linsen[i].Vy = 0.f;
            }
            for(int i=0; i < rohre.length; i++) {
              rohre[i].Ax = 0.f;
              rohre[i].Ay = 0.f;
              rohre[i].Vx = 0.f;
              rohre[i].Vy = 0.f;
            }
            for(int i=0; i < rohre.length; i++) {
              rohre[i].isSelected = true;
            }

            for(int iray = 0; iray < 3; iray++) lines[iray][3].setColor(Color.yellow);

            xl0 = d.width/2 - 100;
            linsen[0].x = (float)(xl0) - linsen[0].bsizex/2;
            linsen[0].y = d.height/2 - linsen[0].bsizey/2;
            xl1 = xl0 + distance1;
            linsen[1].x = (float)(xl1) - linsen[1].bsizex/2;
            linsen[1].y = d.height/2 - linsen[1].bsizey/2;
            xl2 = xl1 + distance2;
            linsen[2].x = (float)(xl2) - linsen[2].bsizex/2;
            linsen[2].y = d.height/2 - linsen[2].bsizey/2;

            rohre[2].x = (float)(linsen[2].x + linsen[2].bsizex/2 - xh);
            rohre[2].y = d.height/2 - rohre[2].height1/2; 
            ih = (int)(xh);

            rohre[0].x = (float)(linsen[0].x + linsen[0].bsizex/2 - gegenweite - rohre[0].width/2);
            rohre[0].y = d.height/2 - rohre[0].height1;
  
            G = rohre[0].height1;
            GDraw = (int)(G);
            for(int iray = 0; iray < 3; iray++) {
                 r = rohre[0].height1*Math.random();
                 rs = 0.;
                 lines[iray][0].setParam(r, rs);
                 ip1x = (int)(rohre[0].x + rohre[0].width/2);
                 ip1y = (int)(d.height/2 - r);
                 ip2x = (int)(linsen[0].x + linsen[0].bsizex/2);
                 r = r + rs*(ip2x - ip1x);
                 ip2y = d.height/2 - (int)(r);
                 lines[iray][0].setLine(ip1x, ip1y, ip2x, ip2y);

                 rs = rs - r/linsen[0].brennweite;
                 lines[iray][1].setParam(r, rs);
                 ip1x = ip2x;
                 ip1y = ip2y;
                 ip2x = (int)(linsen[1].x + linsen[1].bsizex/2);
                 r = r + rs*(ip2x - ip1x);
                 ip2y = d.height/2 - (int)(r);
                 lines[iray][1].setLine(ip1x, ip1y, ip2x,ip2y);

                 rs = rs - r/linsen[1].brennweite;
                 lines[iray][2].setParam(r, rs);
                 ip1x = ip2x;
                 ip1y = ip2y;
                 ip2x = (int)(linsen[2].x + linsen[2].bsizex/2);
                 r = r + rs*(ip2x - ip1x);
                 ip2y = d.height/2 - (int)(r);
                 lines[iray][2].setLine(ip1x, ip1y, ip2x,ip2y);

                 rs = rs - r/linsen[2].brennweite;
                 lines[iray][3].setParam(r, rs);
                 ip1x = ip2x;
                 ip1y = ip2y;
                 ip2x = d.width;
                 r = r + rs*(ip2x - ip1x);
                 ip2y = d.height/2 - (int)(r);
                 lines[iray][3].setLine(ip1x, ip1y, ip2x, ip2y);  
            }                     

            for( int iray = 0; iray < 3; iray++) {
               lines[iray][4].setLine((int)(rohre[2].x+rohre[2].width/2), 
                                      lines[iray][0].ip1y, 
                                      lines[iray][3].ip1x, lines[iray][3].ip1y);
               lines[iray][4].setColor(Color.magenta);
               lines[iray][5].setLine((int)(rohre[2].x+rohre[2].width/2), 
                                      lines[iray][0].ip1y, 
                                      (int)(xl0), lines[iray][0].ip1y);
               lines[iray][5].setColor(Color.magenta);
            } 
            double deter = 0.;
            double x0 = 0.;
            double y0 = 0.;
            for (int iray = 0; iray < 2; iray++) {
               deter = lines[iray+1][3].rs - lines[iray][3].rs;
               x0 += (  (lines[iray][3].r - lines[iray][3].rs*lines[iray][3].ip1x)
                      - (lines[iray+1][3].r -lines[iray+1][3].rs*lines[iray+1][3].ip1x))/deter;
               y0 += (  (lines[iray][3].r - lines[iray][3].rs*lines[iray][3].ip1x)*lines[iray+1][3].rs
                      - (lines[iray+1][3].r -lines[iray+1][3].rs*lines[iray+1][3].ip1x)*lines[iray][3].rs)/deter;
            }
            x0 /= 2.;
            y0 /= 2.;
            bildweite = x0 - (linsen[2].x + linsen[2].bsizex/2);
            rohre[1].x = (float)(x0) - rohre[1].width/2;
            B = -y0;
            BDraw = (int)(B);
            if(B > 0.) {
               rohre[1].y = d.height/2;
               if(BDraw < 3) BDraw = 3;
               rohre[1].height1 = BDraw;
               rohre[1].height2 = rohre[1].height1;
            }
            else {
               if (BDraw > -3) BDraw = -3; 
               rohre[1].y = d.height/2 + BDraw;
               rohre[1].height1 = - (int)(B);
               rohre[1].height2 = rohre[1].height1;
            }
            rohre[1].makeImages((int)(Math.abs(BDraw)), (int)(Math.abs(BDraw)),10);
            for (int i = 0; i < rohre.length; i++) {
              if(rohre[i].x < 0 || rohre[i].x > d.width) rohre[i].isSelected = false;
            }
            active = true;
        }    
    
        public synchronized void stop() {
            thread = null;
        }
    
    
        public void run() {
            Thread me = Thread.currentThread();
            while (thread == me) {
                repaint();
                try {
                    thread.sleep(10);
                } catch (InterruptedException e) { break; }
            }
            thread = null;
        }
    }

        static class Line {
            public double r, rs;
            public int ip1x, ip1y;
            public int ip2x, ip2y;
            public Color color;

            public Line(Color color) {
              this.color = color;
              r = 0.;
              rs = 0.;
              ip1x = 0;
              ip1y = 0;
              ip2x = 0;
              ip2y = 0;
            }
            public void setColor(Color color) {
              this.color = color;
            }
            public void setParam(double r, double rs) {
              this.r = r;
              this.rs = rs;
            }
            public void setLine(int ip1x, int ip1y, int ip2x, int ip2y) {
              this.ip1x = ip1x;
              this.ip1y = ip1y;
              this.ip2x = ip2x;
              this.ip2y = ip2y;
            }
        }

        static class Linse {
        
            public int height, brennweite, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Linse(Color color, int height, int brennweite, int width) {
                this.color = color;
                this.brennweite = brennweite;
                this.width = width;
                makeImages(height, brennweite, width);
            }
        
        
            public void makeImages(int height, int brennweite, int width) {
                this.height = height;
                this.brennweite = brennweite;
                this.width   = width;
                bsizex = width;
                bsizey = height;
                byte[] data = new byte[height * width];
                int maxr = 0;
                int R = (height*height - width*width)/(4*width);
                if(brennweite <0) R = (height*height - 4*width*width/9)/(8*width/3);
                int p = 0;
                int x0 = 0;
                int y0 = height/2;
                for (int Y = 0; Y < height; Y++) {
                    
                    for (int X = 0; X < width; X++) {
                        if(brennweite > 0) {
                          x0 = R;                        
                          if(X > width/2) x0 = width - R;
                        }
                        else {
                          x0 = 2*width/3 + R;
                          if(X < width/2) x0 = width/3 - R; 
                        } 
                        int r = (int)(Math.sqrt((X-width/2.)*(X-width/2)+(Y-height/2)*(Y-height/2)));
                        if (r > maxr) {
                            maxr = r;
                        }
                        if (brennweite > 0) {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) < R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        else {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) > R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Linse class
        static class Rohr {
        
            public int height1, height2, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Rohr(Color color, int height1, int height2, int width) {
                this.color = color;
                makeImages(height1, height2, width);
            }
        
        
            public void makeImages(int height1, int height2, int width) {
                this.height1 = height1;
                this.height2 = height2;
                this.width   = width;
                bsizex = width;
                bsizey = height1;
                byte[] data = new byte[height1 * width];
                int maxr = 0;
                int p = 0;
                for (int Y = 0; Y < height1; Y++) {
                    int x0 = 0;
                    if(Y < height1/2) x0 = (int) (width*Y*2./(height1-height2));
                    else              x0 = (int) (width*(height1-Y)*2./(height1-height2));
                    for (int X = 0; X < width; X++) {
                        int r = 0;
                        if( height1 < width) r = (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5);
                        else if (height1 > width) r = (int) (width/2 - Math.abs(X - width/2.)   + 0.5);
                        else { 
                            r = Math.max( (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5),
                                          (int) (width/2 - Math.abs(X - width/2.)   + 0.5)   );
                            if(X > 0.75*width) r = (int)(r*Math.exp(-0.75*(X-0.75*width)));
                            if(Y > 0.75*width) r = (int)(r*Math.exp(-0.75*(Y-0.75*width)));
                        }
                        if (r > maxr) {
                            maxr = r;
                        }
                        if(X < x0 ) data[p] = r <= 0 ? 1 : (byte) r;
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height1,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Rohr class


    static class DemoControls extends JFrame implements ActionListener {

        Demo demo;

	JButton reset;
        JButton start;
        JButton text;
        JButton label4;
        JComboBox xh;

        public DemoControls(Demo demo) {
            this.demo = demo;
	 
	    resize(800,100);
	    getContentPane().setLayout(new GridLayout(1,5));
            setBackground(Color.gray);
            label4 = new JButton("h =");
	    getContentPane().add(label4);
            getContentPane().add(xh = new JComboBox());
            xh.addItem("10");
            xh.addItem("20");
            xh.addItem("30");
            xh.addItem("40");
            xh.addItem("50");
            xh.addItem("60");
            xh.addItem("70");

            xh.setSelectedIndex(4);
            xh.addActionListener(this);

	    reset = new JButton("Reset");
	    reset.addActionListener(this);
	    getContentPane().add(reset);
	    start = new JButton("Start");
	    start.addActionListener(this);
	    getContentPane().add(start);
	    text = new JButton("Text");
	    text.addActionListener(this);
	    getContentPane().add(text);
        }
         

        


        public void actionPerformed(ActionEvent e) { 
	    JComponent c = (JComponent) e.getSource();

	    
	    if(c== reset)
	    {
	      demo.reset();
	    }
            else if(c == start)
            {
              demo.active = false;
            }
            else if(c == text)
            {
              if(demo.textdraw == false) demo.textdraw = true;
              else demo.textdraw = false;
            }
            else if(c == xh) 
            {
                int size = Integer.parseInt((String) xh.getSelectedItem());
                demo.xh = 5*size;
                demo.reset();
                return;
            }

        }
    } // End DemoControls

} // End MatrizenOptik class
