// Author: Harm Fesefeldt (HarfeSoft) (cc) 2007

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;



public class Mikroskop extends JApplet {

    Demo demo; 				
    DemoControls controls;
    
    public void init() {
        getContentPane().setLayout(new GridLayout(1,1)); 
        resize(1000,600);
        getContentPane().add(demo = new Demo());
        controls = new DemoControls(demo);
	controls.show();
	
    }
	

    public void start() {
	
        demo.start();
    }
  
    public void stop() {
        demo.stop();
    }




    static class Demo extends JPanel implements Runnable {
    
        private static Color colors[] = 
                { Color.blue,Color.red,Color.gray,Color.green, Color.magenta, Color.cyan};
        private Thread thread;
        private BufferedImage bimg;
        private long now, deltaT, lasttime;
        private boolean active;
        private boolean clearDemo = true;
        protected Ball balls[] = new Ball[1];
        protected Linse linsen[] = new Linse[3];
        protected Rohr rohre[] = new Rohr[4];
        public boolean textdraw = false;
        Font font;
        public int dAL, gL;
        int ib1x, ib1y, ib2x, ib2y,ib3x, ib3y, ib4x, ib4y, isx, isy ;
        int i1x, i1y, i2x, i2y, i3x, i3y, i4x, i4y, i5x, i5y, i6x, i6y, i7x, i7y;
        int vergr, tubus, epsilon, vergrth;
        double help1, help2;

        public Demo() {
            setBackground(Color.black);
            
                balls[0] = new Ball(colors[2], 140);
                linsen[0] = new Linse(colors[0], 200, 23, 60);
                linsen[1] = new Linse(colors[3], 400, 80, 40);
                linsen[2] = new Linse(colors[3], 200, 40, 60);

                rohre[0] = new Rohr(colors[1], 20, 20,10);
                rohre[1] = new Rohr(colors[1], 50, 50,10);
                rohre[2] = new Rohr(colors[4], 100,100,10);
                rohre[3] = new Rohr(colors[5],  10, 10,10);
	        for (int i = 0; i < linsen.length; i++) linsen[i].isSelected = true;
	        for (int i = 0; i < balls.length; i++) balls[i].isSelected = true;
	        for (int i = 0; i < rohre.length; i++) rohre[i].isSelected = true;
                font = new Font("Arial",Font.BOLD,18);
                active = true;
                gL = 36;
                dAL = 40;
                tubus = 296;
                epsilon = 5;

            
        }
        
        
        public void step(int w, int h) {
            if (lasttime == 0) {
                lasttime = System.currentTimeMillis();
            }
            now = System.currentTimeMillis();
            deltaT = now - lasttime;
            for (int i = 0; i < balls.length; i++) {
                balls[i].step(deltaT, w, h);
            }
            for (int i = 0; i < linsen.length; i++) {
                linsen[i].step(deltaT, w, h);
            }
            for (int i = 0; i < rohre.length; i++) {
                rohre[i].step(deltaT, w, h);
            }
            if (!active) {
                active = true;
                for (int i = 0; i < balls.length; i++) {
                    balls[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    balls[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    balls[i].Ay = 0.0002f;
                }
                for (int i = 0; i < linsen.length; i++) {
                    linsen[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    linsen[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    linsen[i].Ay = 0.0002f;
                }
                for (int i = 0; i < rohre.length; i++) {
                    rohre[i].Vx = (float)Math.random() / 4.0f - 0.125f;
                    rohre[i].Vy = -(float)Math.random() / 4.0f - 0.2f;
                    rohre[i].Ay = 0.0002f;
                }
            }
        }
    
    
        public void drawDemo(int w, int h, Graphics2D g2) {
            for (int i = 0; i < balls.length; i++) {
                Ball b = balls[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < linsen.length; i++) {
                Linse b = linsen[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            for (int i = 0; i < rohre.length; i++) {
                Rohr b = rohre[i];
                if (b == null || b.imgs[b.index] == null || !b.isSelected) {
                    continue;
                }
                g2.drawImage(b.imgs[b.index], (int) b.x, (int) b.y, this);
            }
            lasttime = now;
        }
    
    
        public Graphics2D createGraphics2D(int w, int h) {
            Graphics2D g2 = null;
            if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
                bimg = (BufferedImage) createImage(w, h);
            } 
            g2 = bimg.createGraphics();
            if (clearDemo) {
                g2.setBackground(getBackground());
                g2.clearRect(0, 0, w, h);
            }
            return g2;
        }
    
    
        public void paint(Graphics g) {
            Dimension d = getSize();
            step(d.width, d.height);
            Graphics2D g2 = createGraphics2D(d.width, d.height);
            drawDemo(d.width, d.height, g2);
            g2.dispose();
            g.drawImage(bimg, 0, 0, this);
            
              g.setColor(Color.white);
              g.drawLine(0, d.height/2, d.width, d.height/2);
              g.drawLine(0, d.height/2+1, d.width, d.height/2+1);
              g.drawLine(0, d.height/2-1, d.width, d.height/2-1);
              for(int i=0; i<3; i++) {
                g.drawLine(ib1x+i-1,ib1y-5,ib1x+i-1,ib1y+5);
                g.drawLine(ib2x+i-1,ib2y-5,ib2x+i-1,ib2y+5);
                g.drawLine(ib3x+i-1,ib3y-5,ib3x+i-1,ib3y+5);
                g.drawLine(ib4x+i-1,ib4y-5,ib4x+i-1,ib4y+5);
                g.drawLine(isx+i-1, isy-5, isx+i-1, isy+5);
              }


              g.setColor(Color.yellow);
              g.drawLine(i1x, i1y, i2x, i2y);
              g.drawLine(i2x, i2y, i3x, i3y);
              g.drawLine(i1x, i1y, i3x, i3y);
              g.drawLine(i5x, i5y, i6x, i6y);
              g.drawLine(i3x, i3y, i4x, i4y);
              g.drawLine(ib2x, ib2y, i6x, i6y);
              g.drawLine(i7x, i7y, i6x, i6y);

              g.setFont(font);
              g.setColor(Color.white);
              g.drawString("H.Fesefeldt (cc) 2007", 25, d.height-25);  
              g.drawString("Vergroesserung  V = " + vergr + " ( " + vergrth + " )", 600, d.height-25);
              g.drawString("S", isx-5, isy + 25);
        }
    
    
        public void start() {
            thread = new Thread(this);
            thread.setPriority(Thread.MIN_PRIORITY);
            reset();
            thread.start();
        }
    
        public void reset() {
            for(int i=0; i<linsen.length;i++) {
             linsen[i].Ax = 0.f;
             linsen[i].Ay = 0.f;
             linsen[i].Vx = 0.f;
             linsen[i].Vy = 0.f;
            }
            balls[0].Ax = 0.f;
            balls[0].Ay = 0.f;
            balls[0].Vx = 0.f;
            balls[0].Vy = 0.f;
            for(int i=0; i<rohre.length; i++) {
             rohre[i].Ax = 0.f;
             rohre[i].Ay = 0.f;
             rohre[i].Vx = 0.f;
             rohre[i].Vy = 0.f;
            }

            Dimension d = getSize();
            

            balls[0].x = d.width - balls[0].bsize-50;
            balls[0].y = d.height/2 - balls[0].bsize/2;
            linsen[0].x = d.width - balls[0].bsize-50;
            linsen[0].y = d.height/2 - linsen[0].bsizey/2;
            linsen[1].x = d.width - balls[0].bsize - 50 - dAL - linsen[1].bsizex/2;
            linsen[1].y = d.height/2 - linsen[1].bsizey/2;
            linsen[2].x = linsen[1].x + linsen[1].bsizex/2 - linsen[1].brennweite 
                          - tubus - linsen[2].brennweite - linsen[2].bsizex/2;
            linsen[2].y = d.height/2 - linsen[2].bsizey/2;
            rohre[3].x = linsen[2].x + linsen[2].bsizex/2 - linsen[2].brennweite 
                         - epsilon - rohre[2].width/2;
            rohre[3].y = d.height/2 - rohre[3].height1;
            double g = linsen[2].brennweite + epsilon;
            double G = rohre[3].height1;
            double b = Math.abs(linsen[2].brennweite*g/(g - linsen[2].brennweite));
            double B = Math.abs(b*G/g);
            rohre[0].makeImages((int)(B), (int)(B), 10);
            rohre[0].x = linsen[2].x + linsen[2].bsizex/2 + (float)(b) - rohre[0].width/2;
            rohre[0].y = d.height/2;
            double f2 = linsen[0].brennweite;
            double f1 = f2/1.34;
            g = linsen[1].x + linsen[1].bsizex/2 - rohre[0].x - rohre[0].width/2;
            G = rohre[0].height1;
            b = Math.abs(linsen[1].brennweite*g/(g-linsen[1].brennweite));
            B = Math.abs(b*G/g);
            help1 = B/(linsen[0].x + linsen[0].bsizex/2 - rohre[2].x - rohre[2].width/2);
            help2 = rohre[3].height1/250.;
            help1 = Math.atan(help1);
            vergr = (int)(help1/help2);
            rohre[2].makeImages((int)(B),(int)(B),10);
            rohre[2].x = linsen[1].x + linsen[1].bsizex/2 - (float)(b) - rohre[2].width/2;
            rohre[2].y = d.height/2;
            g = b + dAL;
            G = rohre[2].height1;
            double phi1 = G/g;
            double gf1 = g - f1;
            if(gf1 == 0.) gf1 = 1.;
            b = f2*g/gf1;
            B = Math.abs(b*G/g);
            if(B > d.height/20.) B = d.height/20. - 0.5;
            rohre[1].makeImages((int)(10*B), (int)(10*B), 10);          
            rohre[1].x = linsen[0].x + linsen[0].bsizex/2 + (float)(10*b);
            rohre[1].y = d.height/2 -rohre[1].height1;
            active = true;
            vergrth = (int)((tubus+linsen[2].brennweite)*250./(linsen[1].brennweite*linsen[2].brennweite));

            ib1x = (int)(linsen[1].x + linsen[1].bsizex/2 - linsen[1].brennweite);
            ib1y = (int)(d.height/2);
            ib2x = (int)(ib1x + 2*linsen[1].brennweite);
            ib2y = ib1y;
            ib3x = (int)(linsen[2].x + linsen[2].bsizex/2 - linsen[2].brennweite);
            ib3y = (int)(d.height/2);
            ib4x = (int)(ib3x + 2*linsen[2].brennweite);
            ib4y = ib3y;

            i1x = (int)(rohre[3].x + rohre[3].width/2);
            i1y = (int)(rohre[3].y);
            i2x = (int)(linsen[2].x + linsen[2].bsizex/2);
            i2y = i1y;
            i3x = (int)(rohre[0].x + rohre[0].width/2);
            i3y = (int)(d.height/2 + rohre[0].height1);
            i4x = (int)(linsen[1].x + linsen[1].bsizex/2);
            i4y = i3y;
            i5x = i4x;
            i5y = (int)(d.height/2);
            i6x = (int)(rohre[2].x + rohre[2].width/2);
            i6y = (int)(d.height/2 + rohre[2].height1);
            i7x = (int)(rohre[1].x + rohre[1].width/2);
            i7y = (int)(d.height/2 - rohre[1].height1);
            isx = (int)(linsen[0].x + linsen[0].bsizex/2) - 250;
            isy = d.height/2;

            if(rohre[2].height1 > d.height/2) rohre[2].makeImages(d.height/2-2,d.height/2-2,10);
            rohre[2].isSelected = true;
            if(rohre[2].x < 0) rohre[2].isSelected = false;
            if(rohre[2].y < 0) rohre[2].isSelected = false;
            
        }    
    
        public synchronized void stop() {
            thread = null;
        }
    
    
        public void run() {
            Thread me = Thread.currentThread();
            while (thread == me) {
                repaint();
                try {
                    thread.sleep(10);
                } catch (InterruptedException e) { break; }
            }
            thread = null;
        }
    }


        static class Linse {
        
            public int height, brennweite, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Linse(Color color, int height, int brennweite, int width) {
                this.color = color;
                this.height = height;
                this.brennweite = brennweite;
                this.width = width;
                makeImages(height, brennweite, width);
            }
        
        
            public void makeImages(int height, int brennweite, int width) {
                this.height = height;
                this.brennweite = brennweite;
                this.width   = width;
                bsizex = width;
                bsizey = height;
                byte[] data = new byte[height * width];
                int maxr = 0;
                int R = (height*height - width*width)/(4*width);
                if(brennweite <0) R = (height*height - 4*width*width/9)/(8*width/3);
                int p = 0;
                int x0 = 0;
                int y0 = height/2;
                for (int Y = 0; Y < height; Y++) {
                    
                    for (int X = 0; X < width; X++) {
                        if(brennweite > 0) {
                          x0 = R;                        
                          if(X > width/2) x0 = width - R;
                        }
                        else {
                          x0 = 2*width/3 + R;
                          if(X < width/2) x0 = width/3 - R; 
                        } 
                        int r = (int)(Math.sqrt((X-width/2.)*(X-width/2)+(Y-height/2)*(Y-height/2)));
                        if (r > maxr) {
                            maxr = r;
                        }
                        if (brennweite > 0) {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) < R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        else {
                          if((X-x0)*(X-x0)+(Y-y0)*(Y-y0) > R*R ) data[p] = r <= 0 ? 1 : (byte) r;
                        }
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Linse class


        static class Rohr {
        
            public int height1, height2, width;
            public int bsizex, bsizey;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;
        
            public Rohr(Color color, int height1, int height2, int width) {
                this.color = color;
                makeImages(height1, height2, width);
            }
        
        
            public void makeImages(int height1, int height2, int width) {
                this.height1 = height1;
                this.height2 = height2;
                this.width   = width;
                bsizex = width;
                bsizey = height1;
                byte[] data = new byte[height1 * width];
                int maxr = 0;
                int p = 0;
                for (int Y = 0; Y < height1; Y++) {
                    int x0 = 0;
                    if(Y < height1/2) x0 = (int) (width*Y*2./(height1-height2));
                    else              x0 = (int) (width*(height1-Y)*2./(height1-height2));
                    for (int X = 0; X < width; X++) {
                        int r = 0;
                        if( height1 < width) r = (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5);
                        else if (height1 > width) r = (int) (width/2 - Math.abs(X - width/2.)   + 0.5);
                        else { 
                            r = Math.max( (int) (height1/2 - Math.abs(Y - height1/2.) + 0.5),
                                          (int) (width/2 - Math.abs(X - width/2.)   + 0.5)   );
                            if(X > 0.75*width) r = (int)(r*Math.exp(-0.75*(X-0.75*width)));
                            if(Y > 0.75*width) r = (int)(r*Math.exp(-0.75*(Y-0.75*width)));
                        }
                        if (r > maxr) {
                            maxr = r;
                        }
                        if(X < x0 ) data[p] = r <= 0 ? 1 : (byte) r;
                        p++;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        width,height1,width,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsizex >= w) {
                    x = w - bsizex;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsizey >= h) {
                    y = h - bsizey;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Rohr class

        static class Ball {
        
            public int bsize;
            public float x, y;
            public float Vx = 0.1f;
            public float Vy = 0.05f;
            public int nImgs = 5;
            public BufferedImage imgs[];
            public int index = (int) (Math.random() * (nImgs-1));
        
            public float inelasticity = .96f;
            public float Ax = 0.0f;
            public float Ay = 0.0002f;
            private final float Ar = 0.9f;
            private final int UP = 0;
            private final int DOWN = 1;
            private int indexDirection = UP;
            private boolean collision_x, collision_y;
            private float jitter;
            public Color color;
            private boolean isSelected;

        
            public Ball(Color color, int bsize) {
                this.color = color;
                makeImages(bsize);
            }
        
        
            public void makeImages(int bsize) {
                this.bsize = bsize*2;
                int R = bsize;
                byte[] data = new byte[R * 2 * R * 2];
                int maxr = 0;
                for (int Y = 2 * R; --Y >= 0;) {
                    int x0 = (int) (Math.sqrt(R * R - (Y - R) * (Y - R)) + 0.5);
                    int p = Y * (R * 2) + R - x0;
                    for (int X = -x0; X < x0; X++) {
                        int x = X + 15;
                        int y = Y - R + 15;
                        int r = (int) (Math.sqrt(x * x + y * y) + 0.5);
                        if (r > maxr) {
                            maxr = r;
                        }
                        data[p++] = r <= 0 ? 1 : (byte) r;
                    }
                }
        
                imgs = new BufferedImage[nImgs];
        
                int bg = 255;
                byte red[] = new byte[256];
                red[0] = (byte) bg;
                byte green[] = new byte[256];
                green[0] = (byte) bg;
                byte blue[] = new byte[256];
                blue[0] = (byte) bg;

                // for each image, set its color
                for (int r = 0; r < imgs.length; r++) {
                    float b = 0.5f + (float) ((r+1f)/imgs.length/2f);
                    for (int i = maxr; i >= 1; --i) {
                        float d = (float) i / maxr;
                        red[i] = (byte) blend(blend(color.getRed(), 255, d), bg, b);
                        green[i] = (byte) blend(blend(color.getGreen(), 255, d), bg, b);
                        blue[i] = (byte) blend(blend(color.getBlue(), 255, d), bg, b);
                    }
                    IndexColorModel icm = new IndexColorModel(8, maxr + 1,
                                red, green, blue, 0);
                    DataBufferByte dbb = new DataBufferByte(data, data.length);
                    int bandOffsets[] = {0};
                    WritableRaster wr = Raster.createInterleavedRaster(dbb,
                        R*2,R*2,R*2,1, bandOffsets,null);
                    imgs[r] = new BufferedImage(icm, wr,icm.isAlphaPremultiplied(),null);
                }
            }

        
	    // performs the blending of white and color
            private final int blend(int fg, int bg, float fgfactor) {
                return (int) (bg + (fg - bg) * fgfactor);
            }
        

	    // performs the action of bouncing off the window "walls" 
            public void step(long deltaT, int w, int h) {
                collision_x = false;
                collision_y = false;
        
                jitter = (float) Math.random() * .01f - .005f;
        
                x += Vx * deltaT + (Ax / 2.0) * deltaT * deltaT;
                y += Vy * deltaT + (Ay / 2.0) * deltaT * deltaT;
                if (x <= 0.0f) {
                    x = 0.0f;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (x + bsize >= w) {
                    x = w - bsize;
                    Vx = -Vx * inelasticity + jitter;
                    collision_x = true;
                }
                if (y <= 0) {
                    y = 0;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                if (y + bsize >= h) {
                    y = h - bsize;
                    Vx *= inelasticity;
                    Vy = -Vy * inelasticity + jitter;
                    collision_y = true;
                }
                Vy = Vy + Ay * deltaT;
                Vx = Vx + Ax * deltaT;
        
                if (indexDirection == UP) {
                    index++; 
                }
                if (indexDirection == DOWN) {
                    --index; 
                }
                if (index+1 == nImgs) {
                    indexDirection = DOWN;
                }
                if (index == 0) {
                    indexDirection = UP;
                }
            }
        }  // End Ball class


    static class DemoControls extends JFrame implements ActionListener {

        Demo demo;

	JButton reset;
        JButton start;
        JButton label1;
        JButton label2;
        JButton label3;
        JComboBox dtu;
        JComboBox fau;
        JComboBox dal;


        public DemoControls(Demo demo) {
            this.demo = demo;
	 
	    resize(1000,100);
	    getContentPane().setLayout(new GridLayout(1,6));
            setBackground(Color.gray);
            label1 = new JButton("Tubus =");
	    getContentPane().add(label1);
            getContentPane().add(dtu = new JComboBox());

            dtu.addItem("296");
            dtu.addItem("298");
            dtu.addItem("300");
            dtu.addItem("302");
            dtu.addItem("304");
            dtu.addItem("306");
            dtu.addItem("308");
            dtu.addItem("310");
            dtu.addItem("312");
            dtu.addItem("314");
            dtu.addItem("316");
            dtu.addItem("318");

            dtu.setSelectedIndex(0);
            dtu.addActionListener(this);
            label2 = new JButton("f(Auge) =");
	    getContentPane().add(label2);
            getContentPane().add(fau = new JComboBox());
            fau.addItem("17");
            fau.addItem("18");
            fau.addItem("19");
            fau.addItem("20");
            fau.addItem("21");
            fau.addItem("22");
            fau.addItem("23");
            fau.addItem("24");
            fau.addItem("25");
            fau.setSelectedIndex(6);
            fau.addActionListener(this);
            label3 = new JButton("d(Auge-Lupe)=");
	    getContentPane().add(label3);
            getContentPane().add(dal = new JComboBox());
            dal.addItem("120");
            dal.addItem("100");
            dal.addItem("80");
            dal.addItem("60");
            dal.addItem("40");
            dal.addItem("20");
            dal.addItem("10");
            dal.addItem("0");
            dal.setSelectedIndex(4);
            dal.addActionListener(this);

   	    start = new JButton("Start");
	    start.addActionListener(this);
	    getContentPane().add(start);
	    reset = new JButton("Reset");
	    reset.addActionListener(this);
	    getContentPane().add(reset);
        }
         

        


        public void actionPerformed(ActionEvent e) { 
	    JComponent c = (JComponent) e.getSource();

	    
	    if(c== reset)
	    {
	      demo.reset();
	    }
            else if(c == start)
            {
              demo.active = false;
            } 
            else if(c == dtu) 
            {
                int size = Integer.parseInt((String) dtu.getSelectedItem());
                demo.tubus = size;
                demo.reset();
                return;
            }
            else if(c == fau) 
            {
                int size = Integer.parseInt((String) fau.getSelectedItem());
                demo.linsen[0].brennweite = size;
                demo.linsen[0].makeImages(200, size, 83 - size);
                demo.reset();
                return;
            }   
            else if(c == dal) 
            {
                int size = Integer.parseInt((String) dal.getSelectedItem());
                demo.dAL = size;
                demo.reset();
                return;
            }   
        }
    } // End DemoControls

} // End Auge class
